// SPDX-License-Identifier: MIT

use askama::Template;
use axum::{
    extract::Form,
    response::{IntoResponse, Redirect},
    routing::{get, post},
    Router,
};
use serde::Deserialize;

use crate::common::{self, container::ContainerInfo, html_template::HtmlTemplate, CheckAuth};
use crate::error::PageResult;

#[cfg(feature = "restapi")]
mod restapi;

#[derive(Deserialize)]
struct ContainerParam {
    name: String,
}

#[derive(Template)]
#[template(path = "../src/container/templates/container.html")]
struct ContainerTemplate {
    container_info: ContainerInfo,
}

#[derive(Template)]
#[template(path = "../src/container/templates/container_logs.html")]
struct ContainerLogsTemplate {
    container_name: String,
    container_logs: String,
}

pub trait ContainerRoutes {
    fn container_routes(self) -> Self;
}

impl ContainerRoutes for Router {
    fn container_routes(self) -> Self {
        let routes = self;

        #[cfg(feature = "restapi")]
        let routes = restapi::routes(routes);

        routes
            .route("/container", get(container))
            .route("/container_start", post(container_start))
            .route("/container_stop", post(container_stop))
            .route("/container_log", get(container_log))
    }
}

async fn container(_loggedin: CheckAuth) -> PageResult {
    let container_info = ContainerInfo::get().await?;
    let template = ContainerTemplate { container_info };
    Ok(HtmlTemplate::new("コンテナ管理", template).into_response())
}

async fn container_start(
    _loggedin: CheckAuth,
    Form(container_param): Form<ContainerParam>,
) -> PageResult {
    let args = &["podman.sh", "start", &container_param.name];
    common::exec_command(args).await?;

    Ok(Redirect::to("/container").into_response())
}

async fn container_stop(
    _loggedin: CheckAuth,
    Form(container_param): Form<ContainerParam>,
) -> PageResult {
    let args = &["podman.sh", "stop", &container_param.name];
    common::exec_command(args).await?;

    Ok(Redirect::to("/container").into_response())
}

async fn container_log(
    _loggedin: CheckAuth,
    Form(container_param): Form<ContainerParam>,
) -> PageResult {
    let args = &["podman.sh", "logs_merged", &container_param.name];
    let output = common::exec_command(args).await?;

    let template = ContainerLogsTemplate {
        container_name: container_param.name,
        container_logs: String::from_utf8_lossy(&output.stdout).trim().to_string(),
    };
    Ok(HtmlTemplate::new("コンテナログ", template).into_response())
}
